program test_get_1_outerm

  ! Libraries:
  use jumble, only: csvread, deg_to_rad

  use config_m, only: config
  use derived_types, only: eddy, shpc_slice_handler, null_ssh_contour, &
       missing_speed
  use get_1_outerm_m, only: get_1_outerm
  use input_ssh_m, only: input_ssh
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use write_eddy_m, only: write_eddy

  implicit none

  integer:: date = 0

  real, allocatable:: ssh(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if the grid is periodic
  ! in longitude, else (nlon, nlat). Sea-surface height, in m.

  real, allocatable:: u(:, :), v(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if periodic, else
  ! (nlon, nlat) wind, in m s-1

  integer:: ind_targ_extr(2) = [19, 11]
  ! indices in the two dimensions of target extremum

  real:: innermost_level = 0.2933
  ! level of innermost contour, for each extremum
  
  logical:: cyclone = .true.

  real corner(2)
  ! longitude and latitude of the corner of the whole grid, in rad

  real step(2) ! longitude and latitude steps, in rad

  integer nlon, nlat
  ! size of ssh array in input NetCDF, assuming no repeated point if
  ! the grid is global

  logical periodic ! grid is periodic in longitude
  TYPE(shpc_slice_handler) hshp
  type(eddy) e

  real, allocatable:: outside_points(:, :) ! (2, :) longitude and
  ! latitude, in rad, of all the significant extrema, except the
  ! target extremum

  logical:: grid_lon_lat = .true.

  namelist /main_nml/ ind_targ_extr, innermost_level, cyclone, date, &
       grid_lon_lat

  !----------------------------------------------------------------

  write(unit = *, nml = main_nml)
  print  *, "test_get_1_outerm: Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)
  call config
  call input_ssh(corner, step, nlon, nlat, periodic, ssh, u, v)
  call csvread("outside_points.csv", outside_points, first_r = 3)
  outside_points = transpose(outside_points) * deg_to_rad
  e%extr%coord_proj = ind_targ_extr
  e%extr%coord = corner + (ind_targ_extr - 1) * step
  e%out_cont = get_1_outerm(cyclone, e%extr%coord, innermost_level, &
       outside_points, ssh, corner, step)

  if (e%out_cont%closed) then
     e%extr%ssh = ssh(ind_targ_extr(1), ind_targ_extr(2))
     e%cyclone = cyclone
     e%speed_cont = null_ssh_contour()
     e%max_speed = missing_speed
     e%radius4 = - 1
     e%valid = .true.
     call shpc_create(hshp, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
          grid_lon_lat = grid_lon_lat)
     call write_eddy(e, hshp, date, i = 1)
     write(hshp%unit, fmt = *) 0
     CALL shpc_close(hshp)
     print *, 'test_get_1_outerm: Created "SHPC".'
  else
     print *, "test_get_1_outerm: Could not find an outermost contour."
  end if

end program test_get_1_outerm
